// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: Math.proto
// Protobuf C++ Version: 6.33.5

#ifndef Math_2eproto_2epb_2eh
#define Math_2eproto_2epb_2eh

#include <limits>
#include <string>
#include <type_traits>
#include <utility>

#include "google/protobuf/runtime_version.h"
#if PROTOBUF_VERSION != 6033005
#error "Protobuf C++ gencode is built with an incompatible version of"
#error "Protobuf C++ headers/runtime. See"
#error "https://protobuf.dev/support/cross-version-runtime-guarantee/#cpp"
#endif
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/arena.h"
#include "google/protobuf/arenastring.h"
#include "google/protobuf/generated_message_bases.h"
#include "google/protobuf/generated_message_tctable_decl.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/metadata_lite.h"
#include "google/protobuf/generated_message_reflection.h"
#include "google/protobuf/message.h"
#include "google/protobuf/message_lite.h"
#include "google/protobuf/repeated_field.h"  // IWYU pragma: export
#include "google/protobuf/extension_set.h"  // IWYU pragma: export
#include "google/protobuf/generated_enum_reflection.h"
#include "google/protobuf/unknown_field_set.h"
#include "Map.pb.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"

#define PROTOBUF_INTERNAL_EXPORT_Math_2eproto

namespace google {
namespace protobuf {
namespace internal {
template <typename T>
::absl::string_view GetAnyMessageName();
}  // namespace internal
}  // namespace protobuf
}  // namespace google

// Internal implementation detail -- do not use these members.
struct TableStruct_Math_2eproto {
  static const ::uint32_t offsets[];
};
extern "C" {
extern const ::google::protobuf::internal::DescriptorTable descriptor_table_Math_2eproto;
}  // extern "C"
namespace mojo {
namespace spec {
enum KMeansType : int;
extern const uint32_t KMeansType_internal_data_[];
enum NormType : int;
extern const uint32_t NormType_internal_data_[];
class ClipOp;
struct ClipOpDefaultTypeInternal;
extern ClipOpDefaultTypeInternal _ClipOp_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull ClipOp_class_data_;
class ExpM1Op;
struct ExpM1OpDefaultTypeInternal;
extern ExpM1OpDefaultTypeInternal _ExpM1Op_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull ExpM1Op_class_data_;
class ExpOp;
struct ExpOpDefaultTypeInternal;
extern ExpOpDefaultTypeInternal _ExpOp_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull ExpOp_class_data_;
class KMeansOp;
struct KMeansOpDefaultTypeInternal;
extern KMeansOpDefaultTypeInternal _KMeansOp_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull KMeansOp_class_data_;
class LogOp;
struct LogOpDefaultTypeInternal;
extern LogOpDefaultTypeInternal _LogOp_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull LogOp_class_data_;
class MatrixProductOp;
struct MatrixProductOpDefaultTypeInternal;
extern MatrixProductOpDefaultTypeInternal _MatrixProductOp_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull MatrixProductOp_class_data_;
class MedianOp;
struct MedianOpDefaultTypeInternal;
extern MedianOpDefaultTypeInternal _MedianOp_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull MedianOp_class_data_;
class MomentOp;
struct MomentOpDefaultTypeInternal;
extern MomentOpDefaultTypeInternal _MomentOp_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull MomentOp_class_data_;
class NormalizeOp;
struct NormalizeOpDefaultTypeInternal;
extern NormalizeOpDefaultTypeInternal _NormalizeOp_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull NormalizeOp_class_data_;
class PowerOp;
struct PowerOpDefaultTypeInternal;
extern PowerOpDefaultTypeInternal _PowerOp_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull PowerOp_class_data_;
class ScaleOp;
struct ScaleOpDefaultTypeInternal;
extern ScaleOpDefaultTypeInternal _ScaleOp_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull ScaleOp_class_data_;
class SigmoidOp;
struct SigmoidOpDefaultTypeInternal;
extern SigmoidOpDefaultTypeInternal _SigmoidOp_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull SigmoidOp_class_data_;
class SoftmaxOp;
struct SoftmaxOpDefaultTypeInternal;
extern SoftmaxOpDefaultTypeInternal _SoftmaxOp_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull SoftmaxOp_class_data_;
class SqrtOp;
struct SqrtOpDefaultTypeInternal;
extern SqrtOpDefaultTypeInternal _SqrtOp_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull SqrtOp_class_data_;
}  // namespace spec
}  // namespace mojo
namespace google {
namespace protobuf {
template <>
internal::EnumTraitsT<::mojo::spec::KMeansType_internal_data_>
    internal::EnumTraitsImpl::value<::mojo::spec::KMeansType>;
template <>
internal::EnumTraitsT<::mojo::spec::NormType_internal_data_>
    internal::EnumTraitsImpl::value<::mojo::spec::NormType>;
}  // namespace protobuf
}  // namespace google

namespace mojo {
namespace spec {
enum NormType : int {
  UNKNOWN_NORM = 0,
  L1_NORM = 1,
  L2_NORM = 2,
  MAX_NORM = 3,
  NormType_INT_MIN_SENTINEL_DO_NOT_USE_ =
      ::std::numeric_limits<::int32_t>::min(),
  NormType_INT_MAX_SENTINEL_DO_NOT_USE_ =
      ::std::numeric_limits<::int32_t>::max(),
};

extern const uint32_t NormType_internal_data_[];
inline constexpr NormType NormType_MIN =
    static_cast<NormType>(0);
inline constexpr NormType NormType_MAX =
    static_cast<NormType>(3);
inline bool NormType_IsValid(int value) {
  return 0 <= value && value <= 3;
}
inline constexpr int NormType_ARRAYSIZE = 3 + 1;
const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL NormType_descriptor();
template <typename T>
const ::std::string& NormType_Name(T value) {
  static_assert(::std::is_same<T, NormType>::value ||
                    ::std::is_integral<T>::value,
                "Incorrect type passed to NormType_Name().");
  return NormType_Name(static_cast<NormType>(value));
}
template <>
inline const ::std::string& NormType_Name(NormType value) {
  return ::google::protobuf::internal::NameOfDenseEnum<NormType_descriptor, 0, 3>(
      static_cast<int>(value));
}
inline bool NormType_Parse(
    ::absl::string_view name, NormType* PROTOBUF_NONNULL value) {
  return ::google::protobuf::internal::ParseNamedEnum<NormType>(NormType_descriptor(), name,
                                           value);
}
enum KMeansType : int {
  UNKNOWN_KMEANS_TYPE = 0,
  DISTANCE = 1,
  LABEL = 2,
  KMeansType_INT_MIN_SENTINEL_DO_NOT_USE_ =
      ::std::numeric_limits<::int32_t>::min(),
  KMeansType_INT_MAX_SENTINEL_DO_NOT_USE_ =
      ::std::numeric_limits<::int32_t>::max(),
};

extern const uint32_t KMeansType_internal_data_[];
inline constexpr KMeansType KMeansType_MIN =
    static_cast<KMeansType>(0);
inline constexpr KMeansType KMeansType_MAX =
    static_cast<KMeansType>(2);
inline bool KMeansType_IsValid(int value) {
  return 0 <= value && value <= 2;
}
inline constexpr int KMeansType_ARRAYSIZE = 2 + 1;
const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL KMeansType_descriptor();
template <typename T>
const ::std::string& KMeansType_Name(T value) {
  static_assert(::std::is_same<T, KMeansType>::value ||
                    ::std::is_integral<T>::value,
                "Incorrect type passed to KMeansType_Name().");
  return KMeansType_Name(static_cast<KMeansType>(value));
}
template <>
inline const ::std::string& KMeansType_Name(KMeansType value) {
  return ::google::protobuf::internal::NameOfDenseEnum<KMeansType_descriptor, 0, 2>(
      static_cast<int>(value));
}
inline bool KMeansType_Parse(
    ::absl::string_view name, KMeansType* PROTOBUF_NONNULL value) {
  return ::google::protobuf::internal::ParseNamedEnum<KMeansType>(KMeansType_descriptor(), name,
                                           value);
}

// ===================================================================


// -------------------------------------------------------------------

class SqrtOp final : public ::google::protobuf::internal::ZeroFieldsBase
/* @@protoc_insertion_point(class_definition:mojo.spec.SqrtOp) */ {
 public:
  inline SqrtOp() : SqrtOp(nullptr) {}

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(SqrtOp* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(SqrtOp));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR SqrtOp(::google::protobuf::internal::ConstantInitialized);

  inline SqrtOp(const SqrtOp& from) : SqrtOp(nullptr, from) {}
  inline SqrtOp(SqrtOp&& from) noexcept
      : SqrtOp(nullptr, ::std::move(from)) {}
  inline SqrtOp& operator=(const SqrtOp& from) {
    CopyFrom(from);
    return *this;
  }
  inline SqrtOp& operator=(SqrtOp&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const SqrtOp& default_instance() {
    return *reinterpret_cast<const SqrtOp*>(
        &_SqrtOp_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 5;
  friend void swap(SqrtOp& a, SqrtOp& b) { a.Swap(&b); }
  inline void Swap(SqrtOp* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(SqrtOp* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  SqrtOp* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::internal::ZeroFieldsBase::DefaultConstruct<SqrtOp>(arena);
  }
  using ::google::protobuf::internal::ZeroFieldsBase::CopyFrom;
  inline void CopyFrom(const SqrtOp& from) {
    ::google::protobuf::internal::ZeroFieldsBase::CopyImpl(*this, from);
  }
  using ::google::protobuf::internal::ZeroFieldsBase::MergeFrom;
  void MergeFrom(const SqrtOp& from) {
    ::google::protobuf::internal::ZeroFieldsBase::MergeImpl(*this, from);
  }

  public:
  bool IsInitialized() const {
    return true;
  }
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "mojo.spec.SqrtOp"; }

  explicit SqrtOp(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  SqrtOp(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const SqrtOp& from);
  SqrtOp(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, SqrtOp&& from) noexcept
      : SqrtOp(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  // @@protoc_insertion_point(class_scope:mojo.spec.SqrtOp)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<0, 0,
                                   0, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  friend struct ::TableStruct_Math_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull SqrtOp_class_data_;
// -------------------------------------------------------------------

class SoftmaxOp final : public ::google::protobuf::internal::ZeroFieldsBase
/* @@protoc_insertion_point(class_definition:mojo.spec.SoftmaxOp) */ {
 public:
  inline SoftmaxOp() : SoftmaxOp(nullptr) {}

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(SoftmaxOp* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(SoftmaxOp));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR SoftmaxOp(::google::protobuf::internal::ConstantInitialized);

  inline SoftmaxOp(const SoftmaxOp& from) : SoftmaxOp(nullptr, from) {}
  inline SoftmaxOp(SoftmaxOp&& from) noexcept
      : SoftmaxOp(nullptr, ::std::move(from)) {}
  inline SoftmaxOp& operator=(const SoftmaxOp& from) {
    CopyFrom(from);
    return *this;
  }
  inline SoftmaxOp& operator=(SoftmaxOp&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const SoftmaxOp& default_instance() {
    return *reinterpret_cast<const SoftmaxOp*>(
        &_SoftmaxOp_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 6;
  friend void swap(SoftmaxOp& a, SoftmaxOp& b) { a.Swap(&b); }
  inline void Swap(SoftmaxOp* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(SoftmaxOp* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  SoftmaxOp* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::internal::ZeroFieldsBase::DefaultConstruct<SoftmaxOp>(arena);
  }
  using ::google::protobuf::internal::ZeroFieldsBase::CopyFrom;
  inline void CopyFrom(const SoftmaxOp& from) {
    ::google::protobuf::internal::ZeroFieldsBase::CopyImpl(*this, from);
  }
  using ::google::protobuf::internal::ZeroFieldsBase::MergeFrom;
  void MergeFrom(const SoftmaxOp& from) {
    ::google::protobuf::internal::ZeroFieldsBase::MergeImpl(*this, from);
  }

  public:
  bool IsInitialized() const {
    return true;
  }
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "mojo.spec.SoftmaxOp"; }

  explicit SoftmaxOp(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  SoftmaxOp(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const SoftmaxOp& from);
  SoftmaxOp(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, SoftmaxOp&& from) noexcept
      : SoftmaxOp(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  // @@protoc_insertion_point(class_scope:mojo.spec.SoftmaxOp)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<0, 0,
                                   0, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  friend struct ::TableStruct_Math_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull SoftmaxOp_class_data_;
// -------------------------------------------------------------------

class SigmoidOp final : public ::google::protobuf::internal::ZeroFieldsBase
/* @@protoc_insertion_point(class_definition:mojo.spec.SigmoidOp) */ {
 public:
  inline SigmoidOp() : SigmoidOp(nullptr) {}

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(SigmoidOp* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(SigmoidOp));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR SigmoidOp(::google::protobuf::internal::ConstantInitialized);

  inline SigmoidOp(const SigmoidOp& from) : SigmoidOp(nullptr, from) {}
  inline SigmoidOp(SigmoidOp&& from) noexcept
      : SigmoidOp(nullptr, ::std::move(from)) {}
  inline SigmoidOp& operator=(const SigmoidOp& from) {
    CopyFrom(from);
    return *this;
  }
  inline SigmoidOp& operator=(SigmoidOp&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const SigmoidOp& default_instance() {
    return *reinterpret_cast<const SigmoidOp*>(
        &_SigmoidOp_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 2;
  friend void swap(SigmoidOp& a, SigmoidOp& b) { a.Swap(&b); }
  inline void Swap(SigmoidOp* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(SigmoidOp* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  SigmoidOp* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::internal::ZeroFieldsBase::DefaultConstruct<SigmoidOp>(arena);
  }
  using ::google::protobuf::internal::ZeroFieldsBase::CopyFrom;
  inline void CopyFrom(const SigmoidOp& from) {
    ::google::protobuf::internal::ZeroFieldsBase::CopyImpl(*this, from);
  }
  using ::google::protobuf::internal::ZeroFieldsBase::MergeFrom;
  void MergeFrom(const SigmoidOp& from) {
    ::google::protobuf::internal::ZeroFieldsBase::MergeImpl(*this, from);
  }

  public:
  bool IsInitialized() const {
    return true;
  }
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "mojo.spec.SigmoidOp"; }

  explicit SigmoidOp(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  SigmoidOp(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const SigmoidOp& from);
  SigmoidOp(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, SigmoidOp&& from) noexcept
      : SigmoidOp(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  // @@protoc_insertion_point(class_scope:mojo.spec.SigmoidOp)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<0, 0,
                                   0, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  friend struct ::TableStruct_Math_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull SigmoidOp_class_data_;
// -------------------------------------------------------------------

class ScaleOp final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:mojo.spec.ScaleOp) */ {
 public:
  inline ScaleOp() : ScaleOp(nullptr) {}
  ~ScaleOp() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ScaleOp* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ScaleOp));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR ScaleOp(::google::protobuf::internal::ConstantInitialized);

  inline ScaleOp(const ScaleOp& from) : ScaleOp(nullptr, from) {}
  inline ScaleOp(ScaleOp&& from) noexcept
      : ScaleOp(nullptr, ::std::move(from)) {}
  inline ScaleOp& operator=(const ScaleOp& from) {
    CopyFrom(from);
    return *this;
  }
  inline ScaleOp& operator=(ScaleOp&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const ScaleOp& default_instance() {
    return *reinterpret_cast<const ScaleOp*>(
        &_ScaleOp_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 13;
  friend void swap(ScaleOp& a, ScaleOp& b) { a.Swap(&b); }
  inline void Swap(ScaleOp* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ScaleOp* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  ScaleOp* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<ScaleOp>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const ScaleOp& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const ScaleOp& from) { ScaleOp::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(ScaleOp* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "mojo.spec.ScaleOp"; }

  explicit ScaleOp(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ScaleOp(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ScaleOp& from);
  ScaleOp(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ScaleOp&& from) noexcept
      : ScaleOp(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kMeanFieldNumber = 1,
    kScaleFieldNumber = 2,
  };
  // double mean = 1;
  void clear_mean() ;
  double mean() const;
  void set_mean(double value);

  private:
  double _internal_mean() const;
  void _internal_set_mean(double value);

  public:
  // double scale = 2;
  void clear_scale() ;
  double scale() const;
  void set_scale(double value);

  private:
  double _internal_scale() const;
  void _internal_set_scale(double value);

  public:
  // @@protoc_insertion_point(class_scope:mojo.spec.ScaleOp)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<1, 2,
                                   0, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const ScaleOp& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    double mean_;
    double scale_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_Math_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull ScaleOp_class_data_;
// -------------------------------------------------------------------

class PowerOp final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:mojo.spec.PowerOp) */ {
 public:
  inline PowerOp() : PowerOp(nullptr) {}
  ~PowerOp() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(PowerOp* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(PowerOp));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR PowerOp(::google::protobuf::internal::ConstantInitialized);

  inline PowerOp(const PowerOp& from) : PowerOp(nullptr, from) {}
  inline PowerOp(PowerOp&& from) noexcept
      : PowerOp(nullptr, ::std::move(from)) {}
  inline PowerOp& operator=(const PowerOp& from) {
    CopyFrom(from);
    return *this;
  }
  inline PowerOp& operator=(PowerOp&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const PowerOp& default_instance() {
    return *reinterpret_cast<const PowerOp*>(
        &_PowerOp_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 3;
  friend void swap(PowerOp& a, PowerOp& b) { a.Swap(&b); }
  inline void Swap(PowerOp* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(PowerOp* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  PowerOp* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<PowerOp>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const PowerOp& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const PowerOp& from) { PowerOp::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(PowerOp* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "mojo.spec.PowerOp"; }

  explicit PowerOp(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  PowerOp(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const PowerOp& from);
  PowerOp(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, PowerOp&& from) noexcept
      : PowerOp(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kExpFieldNumber = 1,
  };
  // double exp = 1;
  void clear_exp() ;
  double exp() const;
  void set_exp(double value);

  private:
  double _internal_exp() const;
  void _internal_set_exp(double value);

  public:
  // @@protoc_insertion_point(class_scope:mojo.spec.PowerOp)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<0, 1,
                                   0, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const PowerOp& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    double exp_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_Math_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull PowerOp_class_data_;
// -------------------------------------------------------------------

class NormalizeOp final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:mojo.spec.NormalizeOp) */ {
 public:
  inline NormalizeOp() : NormalizeOp(nullptr) {}
  ~NormalizeOp() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(NormalizeOp* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(NormalizeOp));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR NormalizeOp(::google::protobuf::internal::ConstantInitialized);

  inline NormalizeOp(const NormalizeOp& from) : NormalizeOp(nullptr, from) {}
  inline NormalizeOp(NormalizeOp&& from) noexcept
      : NormalizeOp(nullptr, ::std::move(from)) {}
  inline NormalizeOp& operator=(const NormalizeOp& from) {
    CopyFrom(from);
    return *this;
  }
  inline NormalizeOp& operator=(NormalizeOp&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const NormalizeOp& default_instance() {
    return *reinterpret_cast<const NormalizeOp*>(
        &_NormalizeOp_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 9;
  friend void swap(NormalizeOp& a, NormalizeOp& b) { a.Swap(&b); }
  inline void Swap(NormalizeOp* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(NormalizeOp* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  NormalizeOp* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<NormalizeOp>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const NormalizeOp& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const NormalizeOp& from) { NormalizeOp::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(NormalizeOp* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "mojo.spec.NormalizeOp"; }

  explicit NormalizeOp(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  NormalizeOp(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const NormalizeOp& from);
  NormalizeOp(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, NormalizeOp&& from) noexcept
      : NormalizeOp(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kTypeFieldNumber = 1,
  };
  // .mojo.spec.NormType type = 1;
  void clear_type() ;
  ::mojo::spec::NormType type() const;
  void set_type(::mojo::spec::NormType value);

  private:
  ::mojo::spec::NormType _internal_type() const;
  void _internal_set_type(::mojo::spec::NormType value);

  public:
  // @@protoc_insertion_point(class_scope:mojo.spec.NormalizeOp)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<0, 1,
                                   0, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const NormalizeOp& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    int type_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_Math_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull NormalizeOp_class_data_;
// -------------------------------------------------------------------

class MomentOp final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:mojo.spec.MomentOp) */ {
 public:
  inline MomentOp() : MomentOp(nullptr) {}
  ~MomentOp() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(MomentOp* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(MomentOp));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR MomentOp(::google::protobuf::internal::ConstantInitialized);

  inline MomentOp(const MomentOp& from) : MomentOp(nullptr, from) {}
  inline MomentOp(MomentOp&& from) noexcept
      : MomentOp(nullptr, ::std::move(from)) {}
  inline MomentOp& operator=(const MomentOp& from) {
    CopyFrom(from);
    return *this;
  }
  inline MomentOp& operator=(MomentOp&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const MomentOp& default_instance() {
    return *reinterpret_cast<const MomentOp*>(
        &_MomentOp_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 8;
  friend void swap(MomentOp& a, MomentOp& b) { a.Swap(&b); }
  inline void Swap(MomentOp* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(MomentOp* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  MomentOp* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<MomentOp>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const MomentOp& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const MomentOp& from) { MomentOp::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(MomentOp* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "mojo.spec.MomentOp"; }

  explicit MomentOp(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  MomentOp(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const MomentOp& from);
  MomentOp(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, MomentOp&& from) noexcept
      : MomentOp(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kOrderFieldNumber = 1,
  };
  // int32 order = 1;
  void clear_order() ;
  ::int32_t order() const;
  void set_order(::int32_t value);

  private:
  ::int32_t _internal_order() const;
  void _internal_set_order(::int32_t value);

  public:
  // @@protoc_insertion_point(class_scope:mojo.spec.MomentOp)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<0, 1,
                                   0, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const MomentOp& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::int32_t order_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_Math_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull MomentOp_class_data_;
// -------------------------------------------------------------------

class MedianOp final : public ::google::protobuf::internal::ZeroFieldsBase
/* @@protoc_insertion_point(class_definition:mojo.spec.MedianOp) */ {
 public:
  inline MedianOp() : MedianOp(nullptr) {}

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(MedianOp* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(MedianOp));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR MedianOp(::google::protobuf::internal::ConstantInitialized);

  inline MedianOp(const MedianOp& from) : MedianOp(nullptr, from) {}
  inline MedianOp(MedianOp&& from) noexcept
      : MedianOp(nullptr, ::std::move(from)) {}
  inline MedianOp& operator=(const MedianOp& from) {
    CopyFrom(from);
    return *this;
  }
  inline MedianOp& operator=(MedianOp&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const MedianOp& default_instance() {
    return *reinterpret_cast<const MedianOp*>(
        &_MedianOp_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 7;
  friend void swap(MedianOp& a, MedianOp& b) { a.Swap(&b); }
  inline void Swap(MedianOp* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(MedianOp* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  MedianOp* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::internal::ZeroFieldsBase::DefaultConstruct<MedianOp>(arena);
  }
  using ::google::protobuf::internal::ZeroFieldsBase::CopyFrom;
  inline void CopyFrom(const MedianOp& from) {
    ::google::protobuf::internal::ZeroFieldsBase::CopyImpl(*this, from);
  }
  using ::google::protobuf::internal::ZeroFieldsBase::MergeFrom;
  void MergeFrom(const MedianOp& from) {
    ::google::protobuf::internal::ZeroFieldsBase::MergeImpl(*this, from);
  }

  public:
  bool IsInitialized() const {
    return true;
  }
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "mojo.spec.MedianOp"; }

  explicit MedianOp(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  MedianOp(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const MedianOp& from);
  MedianOp(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, MedianOp&& from) noexcept
      : MedianOp(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  // @@protoc_insertion_point(class_scope:mojo.spec.MedianOp)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<0, 0,
                                   0, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  friend struct ::TableStruct_Math_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull MedianOp_class_data_;
// -------------------------------------------------------------------

class LogOp final : public ::google::protobuf::internal::ZeroFieldsBase
/* @@protoc_insertion_point(class_definition:mojo.spec.LogOp) */ {
 public:
  inline LogOp() : LogOp(nullptr) {}

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(LogOp* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(LogOp));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR LogOp(::google::protobuf::internal::ConstantInitialized);

  inline LogOp(const LogOp& from) : LogOp(nullptr, from) {}
  inline LogOp(LogOp&& from) noexcept
      : LogOp(nullptr, ::std::move(from)) {}
  inline LogOp& operator=(const LogOp& from) {
    CopyFrom(from);
    return *this;
  }
  inline LogOp& operator=(LogOp&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const LogOp& default_instance() {
    return *reinterpret_cast<const LogOp*>(
        &_LogOp_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 4;
  friend void swap(LogOp& a, LogOp& b) { a.Swap(&b); }
  inline void Swap(LogOp* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(LogOp* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  LogOp* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::internal::ZeroFieldsBase::DefaultConstruct<LogOp>(arena);
  }
  using ::google::protobuf::internal::ZeroFieldsBase::CopyFrom;
  inline void CopyFrom(const LogOp& from) {
    ::google::protobuf::internal::ZeroFieldsBase::CopyImpl(*this, from);
  }
  using ::google::protobuf::internal::ZeroFieldsBase::MergeFrom;
  void MergeFrom(const LogOp& from) {
    ::google::protobuf::internal::ZeroFieldsBase::MergeImpl(*this, from);
  }

  public:
  bool IsInitialized() const {
    return true;
  }
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "mojo.spec.LogOp"; }

  explicit LogOp(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  LogOp(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const LogOp& from);
  LogOp(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, LogOp&& from) noexcept
      : LogOp(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  // @@protoc_insertion_point(class_scope:mojo.spec.LogOp)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<0, 0,
                                   0, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  friend struct ::TableStruct_Math_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull LogOp_class_data_;
// -------------------------------------------------------------------

class ExpOp final : public ::google::protobuf::internal::ZeroFieldsBase
/* @@protoc_insertion_point(class_definition:mojo.spec.ExpOp) */ {
 public:
  inline ExpOp() : ExpOp(nullptr) {}

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ExpOp* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ExpOp));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR ExpOp(::google::protobuf::internal::ConstantInitialized);

  inline ExpOp(const ExpOp& from) : ExpOp(nullptr, from) {}
  inline ExpOp(ExpOp&& from) noexcept
      : ExpOp(nullptr, ::std::move(from)) {}
  inline ExpOp& operator=(const ExpOp& from) {
    CopyFrom(from);
    return *this;
  }
  inline ExpOp& operator=(ExpOp&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const ExpOp& default_instance() {
    return *reinterpret_cast<const ExpOp*>(
        &_ExpOp_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 0;
  friend void swap(ExpOp& a, ExpOp& b) { a.Swap(&b); }
  inline void Swap(ExpOp* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ExpOp* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  ExpOp* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::internal::ZeroFieldsBase::DefaultConstruct<ExpOp>(arena);
  }
  using ::google::protobuf::internal::ZeroFieldsBase::CopyFrom;
  inline void CopyFrom(const ExpOp& from) {
    ::google::protobuf::internal::ZeroFieldsBase::CopyImpl(*this, from);
  }
  using ::google::protobuf::internal::ZeroFieldsBase::MergeFrom;
  void MergeFrom(const ExpOp& from) {
    ::google::protobuf::internal::ZeroFieldsBase::MergeImpl(*this, from);
  }

  public:
  bool IsInitialized() const {
    return true;
  }
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "mojo.spec.ExpOp"; }

  explicit ExpOp(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ExpOp(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ExpOp& from);
  ExpOp(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ExpOp&& from) noexcept
      : ExpOp(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  // @@protoc_insertion_point(class_scope:mojo.spec.ExpOp)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<0, 0,
                                   0, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  friend struct ::TableStruct_Math_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull ExpOp_class_data_;
// -------------------------------------------------------------------

class ExpM1Op final : public ::google::protobuf::internal::ZeroFieldsBase
/* @@protoc_insertion_point(class_definition:mojo.spec.ExpM1Op) */ {
 public:
  inline ExpM1Op() : ExpM1Op(nullptr) {}

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ExpM1Op* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ExpM1Op));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR ExpM1Op(::google::protobuf::internal::ConstantInitialized);

  inline ExpM1Op(const ExpM1Op& from) : ExpM1Op(nullptr, from) {}
  inline ExpM1Op(ExpM1Op&& from) noexcept
      : ExpM1Op(nullptr, ::std::move(from)) {}
  inline ExpM1Op& operator=(const ExpM1Op& from) {
    CopyFrom(from);
    return *this;
  }
  inline ExpM1Op& operator=(ExpM1Op&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const ExpM1Op& default_instance() {
    return *reinterpret_cast<const ExpM1Op*>(
        &_ExpM1Op_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 1;
  friend void swap(ExpM1Op& a, ExpM1Op& b) { a.Swap(&b); }
  inline void Swap(ExpM1Op* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ExpM1Op* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  ExpM1Op* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::internal::ZeroFieldsBase::DefaultConstruct<ExpM1Op>(arena);
  }
  using ::google::protobuf::internal::ZeroFieldsBase::CopyFrom;
  inline void CopyFrom(const ExpM1Op& from) {
    ::google::protobuf::internal::ZeroFieldsBase::CopyImpl(*this, from);
  }
  using ::google::protobuf::internal::ZeroFieldsBase::MergeFrom;
  void MergeFrom(const ExpM1Op& from) {
    ::google::protobuf::internal::ZeroFieldsBase::MergeImpl(*this, from);
  }

  public:
  bool IsInitialized() const {
    return true;
  }
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "mojo.spec.ExpM1Op"; }

  explicit ExpM1Op(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ExpM1Op(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ExpM1Op& from);
  ExpM1Op(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ExpM1Op&& from) noexcept
      : ExpM1Op(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  // @@protoc_insertion_point(class_scope:mojo.spec.ExpM1Op)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<0, 0,
                                   0, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  friend struct ::TableStruct_Math_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull ExpM1Op_class_data_;
// -------------------------------------------------------------------

class ClipOp final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:mojo.spec.ClipOp) */ {
 public:
  inline ClipOp() : ClipOp(nullptr) {}
  ~ClipOp() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ClipOp* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ClipOp));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR ClipOp(::google::protobuf::internal::ConstantInitialized);

  inline ClipOp(const ClipOp& from) : ClipOp(nullptr, from) {}
  inline ClipOp(ClipOp&& from) noexcept
      : ClipOp(nullptr, ::std::move(from)) {}
  inline ClipOp& operator=(const ClipOp& from) {
    CopyFrom(from);
    return *this;
  }
  inline ClipOp& operator=(ClipOp&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const ClipOp& default_instance() {
    return *reinterpret_cast<const ClipOp*>(
        &_ClipOp_default_instance_);
  }
  enum MinValCase {
    kInt32Min = 1,
    kInt64Min = 2,
    kFloat32Min = 3,
    kFloat64Min = 4,
    MIN_VAL_NOT_SET = 0,
  };
  enum MaxValCase {
    kInt32Max = 5,
    kInt64Max = 6,
    kFloat32Max = 7,
    kFloat64Max = 8,
    MAX_VAL_NOT_SET = 0,
  };
  static constexpr int kIndexInFileMessages = 10;
  friend void swap(ClipOp& a, ClipOp& b) { a.Swap(&b); }
  inline void Swap(ClipOp* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ClipOp* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  ClipOp* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<ClipOp>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const ClipOp& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const ClipOp& from) { ClipOp::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(ClipOp* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "mojo.spec.ClipOp"; }

  explicit ClipOp(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ClipOp(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ClipOp& from);
  ClipOp(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ClipOp&& from) noexcept
      : ClipOp(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kInt32MinFieldNumber = 1,
    kInt64MinFieldNumber = 2,
    kFloat32MinFieldNumber = 3,
    kFloat64MinFieldNumber = 4,
    kInt32MaxFieldNumber = 5,
    kInt64MaxFieldNumber = 6,
    kFloat32MaxFieldNumber = 7,
    kFloat64MaxFieldNumber = 8,
  };
  // int32 int32_min = 1;
  bool has_int32_min() const;
  void clear_int32_min() ;
  ::int32_t int32_min() const;
  void set_int32_min(::int32_t value);

  private:
  ::int32_t _internal_int32_min() const;
  void _internal_set_int32_min(::int32_t value);

  public:
  // int64 int64_min = 2;
  bool has_int64_min() const;
  void clear_int64_min() ;
  ::int64_t int64_min() const;
  void set_int64_min(::int64_t value);

  private:
  ::int64_t _internal_int64_min() const;
  void _internal_set_int64_min(::int64_t value);

  public:
  // float float32_min = 3;
  bool has_float32_min() const;
  void clear_float32_min() ;
  float float32_min() const;
  void set_float32_min(float value);

  private:
  float _internal_float32_min() const;
  void _internal_set_float32_min(float value);

  public:
  // double float64_min = 4;
  bool has_float64_min() const;
  void clear_float64_min() ;
  double float64_min() const;
  void set_float64_min(double value);

  private:
  double _internal_float64_min() const;
  void _internal_set_float64_min(double value);

  public:
  // int32 int32_max = 5;
  bool has_int32_max() const;
  void clear_int32_max() ;
  ::int32_t int32_max() const;
  void set_int32_max(::int32_t value);

  private:
  ::int32_t _internal_int32_max() const;
  void _internal_set_int32_max(::int32_t value);

  public:
  // int64 int64_max = 6;
  bool has_int64_max() const;
  void clear_int64_max() ;
  ::int64_t int64_max() const;
  void set_int64_max(::int64_t value);

  private:
  ::int64_t _internal_int64_max() const;
  void _internal_set_int64_max(::int64_t value);

  public:
  // float float32_max = 7;
  bool has_float32_max() const;
  void clear_float32_max() ;
  float float32_max() const;
  void set_float32_max(float value);

  private:
  float _internal_float32_max() const;
  void _internal_set_float32_max(float value);

  public:
  // double float64_max = 8;
  bool has_float64_max() const;
  void clear_float64_max() ;
  double float64_max() const;
  void set_float64_max(double value);

  private:
  double _internal_float64_max() const;
  void _internal_set_float64_max(double value);

  public:
  void clear_min_val();
  MinValCase min_val_case() const;
  void clear_max_val();
  MaxValCase max_val_case() const;
  // @@protoc_insertion_point(class_scope:mojo.spec.ClipOp)
 private:
  class _Internal;
  void set_has_int32_min();
  void set_has_int64_min();
  void set_has_float32_min();
  void set_has_float64_min();
  void set_has_int32_max();
  void set_has_int64_max();
  void set_has_float32_max();
  void set_has_float64_max();
  inline bool has_min_val() const;
  inline void clear_has_min_val();
  inline bool has_max_val() const;
  inline void clear_has_max_val();
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<0, 8,
                                   0, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const ClipOp& from_msg);
    union MinValUnion {
      constexpr MinValUnion() : _constinit_{} {}
      ::google::protobuf::internal::ConstantInitialized _constinit_;
      ::int32_t int32_min_;
      ::int64_t int64_min_;
      float float32_min_;
      double float64_min_;
    } min_val_;
    union MaxValUnion {
      constexpr MaxValUnion() : _constinit_{} {}
      ::google::protobuf::internal::ConstantInitialized _constinit_;
      ::int32_t int32_max_;
      ::int64_t int64_max_;
      float float32_max_;
      double float64_max_;
    } max_val_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::uint32_t _oneof_case_[2];
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_Math_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull ClipOp_class_data_;
// -------------------------------------------------------------------

class MatrixProductOp final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:mojo.spec.MatrixProductOp) */ {
 public:
  inline MatrixProductOp() : MatrixProductOp(nullptr) {}
  ~MatrixProductOp() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(MatrixProductOp* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(MatrixProductOp));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR MatrixProductOp(::google::protobuf::internal::ConstantInitialized);

  inline MatrixProductOp(const MatrixProductOp& from) : MatrixProductOp(nullptr, from) {}
  inline MatrixProductOp(MatrixProductOp&& from) noexcept
      : MatrixProductOp(nullptr, ::std::move(from)) {}
  inline MatrixProductOp& operator=(const MatrixProductOp& from) {
    CopyFrom(from);
    return *this;
  }
  inline MatrixProductOp& operator=(MatrixProductOp&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const MatrixProductOp& default_instance() {
    return *reinterpret_cast<const MatrixProductOp*>(
        &_MatrixProductOp_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 12;
  friend void swap(MatrixProductOp& a, MatrixProductOp& b) { a.Swap(&b); }
  inline void Swap(MatrixProductOp* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(MatrixProductOp* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  MatrixProductOp* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<MatrixProductOp>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const MatrixProductOp& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const MatrixProductOp& from) { MatrixProductOp::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(MatrixProductOp* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "mojo.spec.MatrixProductOp"; }

  explicit MatrixProductOp(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  MatrixProductOp(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const MatrixProductOp& from);
  MatrixProductOp(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, MatrixProductOp&& from) noexcept
      : MatrixProductOp(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kMatrixDataFieldNumber = 1,
  };
  // .mojo.spec.ExternalData matrix_data = 1;
  bool has_matrix_data() const;
  void clear_matrix_data() ;
  const ::mojo::spec::ExternalData& matrix_data() const;
  [[nodiscard]] ::mojo::spec::ExternalData* PROTOBUF_NULLABLE release_matrix_data();
  ::mojo::spec::ExternalData* PROTOBUF_NONNULL mutable_matrix_data();
  void set_allocated_matrix_data(::mojo::spec::ExternalData* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_matrix_data(::mojo::spec::ExternalData* PROTOBUF_NULLABLE value);
  ::mojo::spec::ExternalData* PROTOBUF_NULLABLE unsafe_arena_release_matrix_data();

  private:
  const ::mojo::spec::ExternalData& _internal_matrix_data() const;
  ::mojo::spec::ExternalData* PROTOBUF_NONNULL _internal_mutable_matrix_data();

  public:
  // @@protoc_insertion_point(class_scope:mojo.spec.MatrixProductOp)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<0, 1,
                                   1, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const MatrixProductOp& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::mojo::spec::ExternalData* PROTOBUF_NULLABLE matrix_data_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_Math_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull MatrixProductOp_class_data_;
// -------------------------------------------------------------------

class KMeansOp final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:mojo.spec.KMeansOp) */ {
 public:
  inline KMeansOp() : KMeansOp(nullptr) {}
  ~KMeansOp() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(KMeansOp* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(KMeansOp));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR KMeansOp(::google::protobuf::internal::ConstantInitialized);

  inline KMeansOp(const KMeansOp& from) : KMeansOp(nullptr, from) {}
  inline KMeansOp(KMeansOp&& from) noexcept
      : KMeansOp(nullptr, ::std::move(from)) {}
  inline KMeansOp& operator=(const KMeansOp& from) {
    CopyFrom(from);
    return *this;
  }
  inline KMeansOp& operator=(KMeansOp&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const KMeansOp& default_instance() {
    return *reinterpret_cast<const KMeansOp*>(
        &_KMeansOp_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 11;
  friend void swap(KMeansOp& a, KMeansOp& b) { a.Swap(&b); }
  inline void Swap(KMeansOp* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(KMeansOp* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  KMeansOp* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<KMeansOp>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const KMeansOp& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const KMeansOp& from) { KMeansOp::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(KMeansOp* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "mojo.spec.KMeansOp"; }

  explicit KMeansOp(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  KMeansOp(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const KMeansOp& from);
  KMeansOp(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, KMeansOp&& from) noexcept
      : KMeansOp(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kCentroidsFieldNumber = 3,
    kTypeFieldNumber = 1,
    kKFieldNumber = 2,
  };
  // .mojo.spec.ExternalData centroids = 3;
  bool has_centroids() const;
  void clear_centroids() ;
  const ::mojo::spec::ExternalData& centroids() const;
  [[nodiscard]] ::mojo::spec::ExternalData* PROTOBUF_NULLABLE release_centroids();
  ::mojo::spec::ExternalData* PROTOBUF_NONNULL mutable_centroids();
  void set_allocated_centroids(::mojo::spec::ExternalData* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_centroids(::mojo::spec::ExternalData* PROTOBUF_NULLABLE value);
  ::mojo::spec::ExternalData* PROTOBUF_NULLABLE unsafe_arena_release_centroids();

  private:
  const ::mojo::spec::ExternalData& _internal_centroids() const;
  ::mojo::spec::ExternalData* PROTOBUF_NONNULL _internal_mutable_centroids();

  public:
  // .mojo.spec.KMeansType type = 1;
  void clear_type() ;
  ::mojo::spec::KMeansType type() const;
  void set_type(::mojo::spec::KMeansType value);

  private:
  ::mojo::spec::KMeansType _internal_type() const;
  void _internal_set_type(::mojo::spec::KMeansType value);

  public:
  // int32 k = 2;
  void clear_k() ;
  ::int32_t k() const;
  void set_k(::int32_t value);

  private:
  ::int32_t _internal_k() const;
  void _internal_set_k(::int32_t value);

  public:
  // @@protoc_insertion_point(class_scope:mojo.spec.KMeansOp)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<2, 3,
                                   1, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const KMeansOp& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::mojo::spec::ExternalData* PROTOBUF_NULLABLE centroids_;
    int type_;
    ::int32_t k_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_Math_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull KMeansOp_class_data_;

// ===================================================================




// ===================================================================


#ifdef __GNUC__
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// -------------------------------------------------------------------

// ExpOp

// -------------------------------------------------------------------

// ExpM1Op

// -------------------------------------------------------------------

// SigmoidOp

// -------------------------------------------------------------------

// PowerOp

// double exp = 1;
inline void PowerOp::clear_exp() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.exp_ = 0;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline double PowerOp::exp() const {
  // @@protoc_insertion_point(field_get:mojo.spec.PowerOp.exp)
  return _internal_exp();
}
inline void PowerOp::set_exp(double value) {
  _internal_set_exp(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_set:mojo.spec.PowerOp.exp)
}
inline double PowerOp::_internal_exp() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.exp_;
}
inline void PowerOp::_internal_set_exp(double value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.exp_ = value;
}

// -------------------------------------------------------------------

// LogOp

// -------------------------------------------------------------------

// SqrtOp

// -------------------------------------------------------------------

// SoftmaxOp

// -------------------------------------------------------------------

// MedianOp

// -------------------------------------------------------------------

// MomentOp

// int32 order = 1;
inline void MomentOp::clear_order() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.order_ = 0;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline ::int32_t MomentOp::order() const {
  // @@protoc_insertion_point(field_get:mojo.spec.MomentOp.order)
  return _internal_order();
}
inline void MomentOp::set_order(::int32_t value) {
  _internal_set_order(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_set:mojo.spec.MomentOp.order)
}
inline ::int32_t MomentOp::_internal_order() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.order_;
}
inline void MomentOp::_internal_set_order(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.order_ = value;
}

// -------------------------------------------------------------------

// NormalizeOp

// .mojo.spec.NormType type = 1;
inline void NormalizeOp::clear_type() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.type_ = 0;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline ::mojo::spec::NormType NormalizeOp::type() const {
  // @@protoc_insertion_point(field_get:mojo.spec.NormalizeOp.type)
  return _internal_type();
}
inline void NormalizeOp::set_type(::mojo::spec::NormType value) {
  _internal_set_type(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_set:mojo.spec.NormalizeOp.type)
}
inline ::mojo::spec::NormType NormalizeOp::_internal_type() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return static_cast<::mojo::spec::NormType>(_impl_.type_);
}
inline void NormalizeOp::_internal_set_type(::mojo::spec::NormType value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.type_ = value;
}

// -------------------------------------------------------------------

// ClipOp

// int32 int32_min = 1;
inline bool ClipOp::has_int32_min() const {
  return min_val_case() == kInt32Min;
}
inline void ClipOp::set_has_int32_min() {
  _impl_._oneof_case_[0] = kInt32Min;
}
inline void ClipOp::clear_int32_min() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (min_val_case() == kInt32Min) {
    _impl_.min_val_.int32_min_ = 0;
    clear_has_min_val();
  }
}
inline ::int32_t ClipOp::int32_min() const {
  // @@protoc_insertion_point(field_get:mojo.spec.ClipOp.int32_min)
  return _internal_int32_min();
}
inline void ClipOp::set_int32_min(::int32_t value) {
  if (min_val_case() != kInt32Min) {
    clear_min_val();
    set_has_int32_min();
  }
  _impl_.min_val_.int32_min_ = value;
  // @@protoc_insertion_point(field_set:mojo.spec.ClipOp.int32_min)
}
inline ::int32_t ClipOp::_internal_int32_min() const {
  if (min_val_case() == kInt32Min) {
    return _impl_.min_val_.int32_min_;
  }
  return 0;
}

// int64 int64_min = 2;
inline bool ClipOp::has_int64_min() const {
  return min_val_case() == kInt64Min;
}
inline void ClipOp::set_has_int64_min() {
  _impl_._oneof_case_[0] = kInt64Min;
}
inline void ClipOp::clear_int64_min() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (min_val_case() == kInt64Min) {
    _impl_.min_val_.int64_min_ = ::int64_t{0};
    clear_has_min_val();
  }
}
inline ::int64_t ClipOp::int64_min() const {
  // @@protoc_insertion_point(field_get:mojo.spec.ClipOp.int64_min)
  return _internal_int64_min();
}
inline void ClipOp::set_int64_min(::int64_t value) {
  if (min_val_case() != kInt64Min) {
    clear_min_val();
    set_has_int64_min();
  }
  _impl_.min_val_.int64_min_ = value;
  // @@protoc_insertion_point(field_set:mojo.spec.ClipOp.int64_min)
}
inline ::int64_t ClipOp::_internal_int64_min() const {
  if (min_val_case() == kInt64Min) {
    return _impl_.min_val_.int64_min_;
  }
  return ::int64_t{0};
}

// float float32_min = 3;
inline bool ClipOp::has_float32_min() const {
  return min_val_case() == kFloat32Min;
}
inline void ClipOp::set_has_float32_min() {
  _impl_._oneof_case_[0] = kFloat32Min;
}
inline void ClipOp::clear_float32_min() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (min_val_case() == kFloat32Min) {
    _impl_.min_val_.float32_min_ = 0;
    clear_has_min_val();
  }
}
inline float ClipOp::float32_min() const {
  // @@protoc_insertion_point(field_get:mojo.spec.ClipOp.float32_min)
  return _internal_float32_min();
}
inline void ClipOp::set_float32_min(float value) {
  if (min_val_case() != kFloat32Min) {
    clear_min_val();
    set_has_float32_min();
  }
  _impl_.min_val_.float32_min_ = value;
  // @@protoc_insertion_point(field_set:mojo.spec.ClipOp.float32_min)
}
inline float ClipOp::_internal_float32_min() const {
  if (min_val_case() == kFloat32Min) {
    return _impl_.min_val_.float32_min_;
  }
  return 0;
}

// double float64_min = 4;
inline bool ClipOp::has_float64_min() const {
  return min_val_case() == kFloat64Min;
}
inline void ClipOp::set_has_float64_min() {
  _impl_._oneof_case_[0] = kFloat64Min;
}
inline void ClipOp::clear_float64_min() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (min_val_case() == kFloat64Min) {
    _impl_.min_val_.float64_min_ = 0;
    clear_has_min_val();
  }
}
inline double ClipOp::float64_min() const {
  // @@protoc_insertion_point(field_get:mojo.spec.ClipOp.float64_min)
  return _internal_float64_min();
}
inline void ClipOp::set_float64_min(double value) {
  if (min_val_case() != kFloat64Min) {
    clear_min_val();
    set_has_float64_min();
  }
  _impl_.min_val_.float64_min_ = value;
  // @@protoc_insertion_point(field_set:mojo.spec.ClipOp.float64_min)
}
inline double ClipOp::_internal_float64_min() const {
  if (min_val_case() == kFloat64Min) {
    return _impl_.min_val_.float64_min_;
  }
  return 0;
}

// int32 int32_max = 5;
inline bool ClipOp::has_int32_max() const {
  return max_val_case() == kInt32Max;
}
inline void ClipOp::set_has_int32_max() {
  _impl_._oneof_case_[1] = kInt32Max;
}
inline void ClipOp::clear_int32_max() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (max_val_case() == kInt32Max) {
    _impl_.max_val_.int32_max_ = 0;
    clear_has_max_val();
  }
}
inline ::int32_t ClipOp::int32_max() const {
  // @@protoc_insertion_point(field_get:mojo.spec.ClipOp.int32_max)
  return _internal_int32_max();
}
inline void ClipOp::set_int32_max(::int32_t value) {
  if (max_val_case() != kInt32Max) {
    clear_max_val();
    set_has_int32_max();
  }
  _impl_.max_val_.int32_max_ = value;
  // @@protoc_insertion_point(field_set:mojo.spec.ClipOp.int32_max)
}
inline ::int32_t ClipOp::_internal_int32_max() const {
  if (max_val_case() == kInt32Max) {
    return _impl_.max_val_.int32_max_;
  }
  return 0;
}

// int64 int64_max = 6;
inline bool ClipOp::has_int64_max() const {
  return max_val_case() == kInt64Max;
}
inline void ClipOp::set_has_int64_max() {
  _impl_._oneof_case_[1] = kInt64Max;
}
inline void ClipOp::clear_int64_max() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (max_val_case() == kInt64Max) {
    _impl_.max_val_.int64_max_ = ::int64_t{0};
    clear_has_max_val();
  }
}
inline ::int64_t ClipOp::int64_max() const {
  // @@protoc_insertion_point(field_get:mojo.spec.ClipOp.int64_max)
  return _internal_int64_max();
}
inline void ClipOp::set_int64_max(::int64_t value) {
  if (max_val_case() != kInt64Max) {
    clear_max_val();
    set_has_int64_max();
  }
  _impl_.max_val_.int64_max_ = value;
  // @@protoc_insertion_point(field_set:mojo.spec.ClipOp.int64_max)
}
inline ::int64_t ClipOp::_internal_int64_max() const {
  if (max_val_case() == kInt64Max) {
    return _impl_.max_val_.int64_max_;
  }
  return ::int64_t{0};
}

// float float32_max = 7;
inline bool ClipOp::has_float32_max() const {
  return max_val_case() == kFloat32Max;
}
inline void ClipOp::set_has_float32_max() {
  _impl_._oneof_case_[1] = kFloat32Max;
}
inline void ClipOp::clear_float32_max() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (max_val_case() == kFloat32Max) {
    _impl_.max_val_.float32_max_ = 0;
    clear_has_max_val();
  }
}
inline float ClipOp::float32_max() const {
  // @@protoc_insertion_point(field_get:mojo.spec.ClipOp.float32_max)
  return _internal_float32_max();
}
inline void ClipOp::set_float32_max(float value) {
  if (max_val_case() != kFloat32Max) {
    clear_max_val();
    set_has_float32_max();
  }
  _impl_.max_val_.float32_max_ = value;
  // @@protoc_insertion_point(field_set:mojo.spec.ClipOp.float32_max)
}
inline float ClipOp::_internal_float32_max() const {
  if (max_val_case() == kFloat32Max) {
    return _impl_.max_val_.float32_max_;
  }
  return 0;
}

// double float64_max = 8;
inline bool ClipOp::has_float64_max() const {
  return max_val_case() == kFloat64Max;
}
inline void ClipOp::set_has_float64_max() {
  _impl_._oneof_case_[1] = kFloat64Max;
}
inline void ClipOp::clear_float64_max() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (max_val_case() == kFloat64Max) {
    _impl_.max_val_.float64_max_ = 0;
    clear_has_max_val();
  }
}
inline double ClipOp::float64_max() const {
  // @@protoc_insertion_point(field_get:mojo.spec.ClipOp.float64_max)
  return _internal_float64_max();
}
inline void ClipOp::set_float64_max(double value) {
  if (max_val_case() != kFloat64Max) {
    clear_max_val();
    set_has_float64_max();
  }
  _impl_.max_val_.float64_max_ = value;
  // @@protoc_insertion_point(field_set:mojo.spec.ClipOp.float64_max)
}
inline double ClipOp::_internal_float64_max() const {
  if (max_val_case() == kFloat64Max) {
    return _impl_.max_val_.float64_max_;
  }
  return 0;
}

inline bool ClipOp::has_min_val() const {
  return min_val_case() != MIN_VAL_NOT_SET;
}
inline void ClipOp::clear_has_min_val() {
  _impl_._oneof_case_[0] = MIN_VAL_NOT_SET;
}
inline bool ClipOp::has_max_val() const {
  return max_val_case() != MAX_VAL_NOT_SET;
}
inline void ClipOp::clear_has_max_val() {
  _impl_._oneof_case_[1] = MAX_VAL_NOT_SET;
}
inline ClipOp::MinValCase ClipOp::min_val_case() const {
  return ClipOp::MinValCase(_impl_._oneof_case_[0]);
}
inline ClipOp::MaxValCase ClipOp::max_val_case() const {
  return ClipOp::MaxValCase(_impl_._oneof_case_[1]);
}
// -------------------------------------------------------------------

// KMeansOp

// .mojo.spec.KMeansType type = 1;
inline void KMeansOp::clear_type() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.type_ = 0;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000002U);
}
inline ::mojo::spec::KMeansType KMeansOp::type() const {
  // @@protoc_insertion_point(field_get:mojo.spec.KMeansOp.type)
  return _internal_type();
}
inline void KMeansOp::set_type(::mojo::spec::KMeansType value) {
  _internal_set_type(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_set:mojo.spec.KMeansOp.type)
}
inline ::mojo::spec::KMeansType KMeansOp::_internal_type() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return static_cast<::mojo::spec::KMeansType>(_impl_.type_);
}
inline void KMeansOp::_internal_set_type(::mojo::spec::KMeansType value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.type_ = value;
}

// int32 k = 2;
inline void KMeansOp::clear_k() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.k_ = 0;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000004U);
}
inline ::int32_t KMeansOp::k() const {
  // @@protoc_insertion_point(field_get:mojo.spec.KMeansOp.k)
  return _internal_k();
}
inline void KMeansOp::set_k(::int32_t value) {
  _internal_set_k(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  // @@protoc_insertion_point(field_set:mojo.spec.KMeansOp.k)
}
inline ::int32_t KMeansOp::_internal_k() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.k_;
}
inline void KMeansOp::_internal_set_k(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.k_ = value;
}

// .mojo.spec.ExternalData centroids = 3;
inline bool KMeansOp::has_centroids() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000001U);
  PROTOBUF_ASSUME(!value || _impl_.centroids_ != nullptr);
  return value;
}
inline const ::mojo::spec::ExternalData& KMeansOp::_internal_centroids() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::mojo::spec::ExternalData* p = _impl_.centroids_;
  return p != nullptr ? *p : reinterpret_cast<const ::mojo::spec::ExternalData&>(::mojo::spec::_ExternalData_default_instance_);
}
inline const ::mojo::spec::ExternalData& KMeansOp::centroids() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:mojo.spec.KMeansOp.centroids)
  return _internal_centroids();
}
inline void KMeansOp::unsafe_arena_set_allocated_centroids(
    ::mojo::spec::ExternalData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.centroids_);
  }
  _impl_.centroids_ = reinterpret_cast<::mojo::spec::ExternalData*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:mojo.spec.KMeansOp.centroids)
}
inline ::mojo::spec::ExternalData* PROTOBUF_NULLABLE KMeansOp::release_centroids() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::mojo::spec::ExternalData* released = _impl_.centroids_;
  _impl_.centroids_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::mojo::spec::ExternalData* PROTOBUF_NULLABLE KMeansOp::unsafe_arena_release_centroids() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:mojo.spec.KMeansOp.centroids)

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::mojo::spec::ExternalData* temp = _impl_.centroids_;
  _impl_.centroids_ = nullptr;
  return temp;
}
inline ::mojo::spec::ExternalData* PROTOBUF_NONNULL KMeansOp::_internal_mutable_centroids() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.centroids_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::mojo::spec::ExternalData>(GetArena());
    _impl_.centroids_ = reinterpret_cast<::mojo::spec::ExternalData*>(p);
  }
  return _impl_.centroids_;
}
inline ::mojo::spec::ExternalData* PROTOBUF_NONNULL KMeansOp::mutable_centroids()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::mojo::spec::ExternalData* _msg = _internal_mutable_centroids();
  // @@protoc_insertion_point(field_mutable:mojo.spec.KMeansOp.centroids)
  return _msg;
}
inline void KMeansOp::set_allocated_centroids(::mojo::spec::ExternalData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.centroids_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = reinterpret_cast<::google::protobuf::Message*>(value)->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }

  _impl_.centroids_ = reinterpret_cast<::mojo::spec::ExternalData*>(value);
  // @@protoc_insertion_point(field_set_allocated:mojo.spec.KMeansOp.centroids)
}

// -------------------------------------------------------------------

// MatrixProductOp

// .mojo.spec.ExternalData matrix_data = 1;
inline bool MatrixProductOp::has_matrix_data() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000001U);
  PROTOBUF_ASSUME(!value || _impl_.matrix_data_ != nullptr);
  return value;
}
inline const ::mojo::spec::ExternalData& MatrixProductOp::_internal_matrix_data() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::mojo::spec::ExternalData* p = _impl_.matrix_data_;
  return p != nullptr ? *p : reinterpret_cast<const ::mojo::spec::ExternalData&>(::mojo::spec::_ExternalData_default_instance_);
}
inline const ::mojo::spec::ExternalData& MatrixProductOp::matrix_data() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:mojo.spec.MatrixProductOp.matrix_data)
  return _internal_matrix_data();
}
inline void MatrixProductOp::unsafe_arena_set_allocated_matrix_data(
    ::mojo::spec::ExternalData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.matrix_data_);
  }
  _impl_.matrix_data_ = reinterpret_cast<::mojo::spec::ExternalData*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:mojo.spec.MatrixProductOp.matrix_data)
}
inline ::mojo::spec::ExternalData* PROTOBUF_NULLABLE MatrixProductOp::release_matrix_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::mojo::spec::ExternalData* released = _impl_.matrix_data_;
  _impl_.matrix_data_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::mojo::spec::ExternalData* PROTOBUF_NULLABLE MatrixProductOp::unsafe_arena_release_matrix_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:mojo.spec.MatrixProductOp.matrix_data)

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::mojo::spec::ExternalData* temp = _impl_.matrix_data_;
  _impl_.matrix_data_ = nullptr;
  return temp;
}
inline ::mojo::spec::ExternalData* PROTOBUF_NONNULL MatrixProductOp::_internal_mutable_matrix_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.matrix_data_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::mojo::spec::ExternalData>(GetArena());
    _impl_.matrix_data_ = reinterpret_cast<::mojo::spec::ExternalData*>(p);
  }
  return _impl_.matrix_data_;
}
inline ::mojo::spec::ExternalData* PROTOBUF_NONNULL MatrixProductOp::mutable_matrix_data()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::mojo::spec::ExternalData* _msg = _internal_mutable_matrix_data();
  // @@protoc_insertion_point(field_mutable:mojo.spec.MatrixProductOp.matrix_data)
  return _msg;
}
inline void MatrixProductOp::set_allocated_matrix_data(::mojo::spec::ExternalData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.matrix_data_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = reinterpret_cast<::google::protobuf::Message*>(value)->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }

  _impl_.matrix_data_ = reinterpret_cast<::mojo::spec::ExternalData*>(value);
  // @@protoc_insertion_point(field_set_allocated:mojo.spec.MatrixProductOp.matrix_data)
}

// -------------------------------------------------------------------

// ScaleOp

// double mean = 1;
inline void ScaleOp::clear_mean() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.mean_ = 0;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline double ScaleOp::mean() const {
  // @@protoc_insertion_point(field_get:mojo.spec.ScaleOp.mean)
  return _internal_mean();
}
inline void ScaleOp::set_mean(double value) {
  _internal_set_mean(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_set:mojo.spec.ScaleOp.mean)
}
inline double ScaleOp::_internal_mean() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.mean_;
}
inline void ScaleOp::_internal_set_mean(double value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.mean_ = value;
}

// double scale = 2;
inline void ScaleOp::clear_scale() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.scale_ = 0;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000002U);
}
inline double ScaleOp::scale() const {
  // @@protoc_insertion_point(field_get:mojo.spec.ScaleOp.scale)
  return _internal_scale();
}
inline void ScaleOp::set_scale(double value) {
  _internal_set_scale(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_set:mojo.spec.ScaleOp.scale)
}
inline double ScaleOp::_internal_scale() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.scale_;
}
inline void ScaleOp::_internal_set_scale(double value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.scale_ = value;
}

#ifdef __GNUC__
#pragma GCC diagnostic pop
#endif  // __GNUC__

// @@protoc_insertion_point(namespace_scope)
}  // namespace spec
}  // namespace mojo


namespace google {
namespace protobuf {

template <>
struct is_proto_enum<::mojo::spec::NormType> : std::true_type {};
template <>
inline const EnumDescriptor* PROTOBUF_NONNULL GetEnumDescriptor<::mojo::spec::NormType>() {
  return ::mojo::spec::NormType_descriptor();
}
template <>
struct is_proto_enum<::mojo::spec::KMeansType> : std::true_type {};
template <>
inline const EnumDescriptor* PROTOBUF_NONNULL GetEnumDescriptor<::mojo::spec::KMeansType>() {
  return ::mojo::spec::KMeansType_descriptor();
}

}  // namespace protobuf
}  // namespace google

// @@protoc_insertion_point(global_scope)

#include "google/protobuf/port_undef.inc"

#endif  // Math_2eproto_2epb_2eh
